#!/usr/bin/env ruby

# This is a test for FormatR it uses perl and compares output from perl and ruby

require "format.rb"
include FormatR

def compareOutput (infile, perl, tail)
  #grab some output from perl formats that look like the above
  perl_output = IO.popen("#{perl}").readlines()
  perl_output.collect! {|x| x.chop()}
  tail.each {|i| perl_output.push(i.chop())} if (tail)
  File.open("#{infile}"){|file|
    lines = file.readlines
    lines.each_index {|i|
      line = lines[i].chomp!()
      if (!(line === (perl_output[i])))
        raise "testing error between \n#{infile}>#{line}<\nperl            >#{perl_output[i]}<" 
      end
    }
    if (lines.size != perl_output.size)
      raise "testing error between #{infile} #{lines.size()} and #{perl} #{perl_output.size()}, not the same size" 
    end
  }
end


f = <<DOT   
  @###.#   @##########.###  @##.###| @.### @##.. .
#comment
  num,      num,             snum,    snum, snum
  1234567      1234567.890  -12.340| -12.3 -12.. .
left|@<<<<| right|@>>>>| |center|@||||| | num|@##.#|
     twenty_two,  twenty_two,    twenty_two,  snum
left|22   | right|   22| |center|  22   | num|-12.3|
DOT

top = <<DOT
            Passwd File
      Name   Name   Login   Office   Uid       Gid      Home
------------------------------------------------------
DOT

twenty_two = '22'
num = 1234567.89012455
snum = -12.34

fmt1 = Format.new (f)
fmt2 = Format.new (top)
fmt1.setTop (fmt2)

File.open("format_testfile1", File::CREAT | File::WRONLY | File::TRUNC) { |file|
  fmt1.io = file
  fmt2.io = file
  fmt1.resetPage()
  13.times do
    fmt1.printFormat(binding)
  end
}

compareOutput ("format_testfile1", "./format_test.pl 1", nil)#top)

# Test output from perl script
from_perl =  <<FROM_PERL
Foo   @<<<<  @>>>>  @<<<<   @>>>>>    @##.#####   @# @.###    @##.#.
      name,  name,  login,  office,   uid,      uid, gid,   gid,
safdsdfsdfsdf
    sdfsdfsdfdfs
bar @<<<<  @>>>>  @<<<<   @>>>>>    @|||||+  +@#####+  @<<<<
    name,  name,  login,  office,   uid,      gid,     home
baz ^<<<*   *^<<<* ^<<  @<_^_<< @||| ^#.## ^##.###
    name,    name, name,login, login,  home, uid, uid
\@#\@#
uid,uid
FROM_PERL


name = "Paul Rubel"
login ="rubel@crhc.uiuc.edu"
office = "6/301"
uid = 124.135791357;

fmt3 = Format.new (from_perl) 
File.open("format_testfile2", File::CREAT | File::WRONLY | File::TRUNC) { |file|
  gid = -13
  home = "/home/rubel/"
  fmt3.io = file
  fmt3.setTop(fmt2)
  fmt2.io = file
  20.times do
    fmt3.printFormat(binding)
  end
}
raise "line numbering is broken #{fmt3.pageNumber()}" unless (3 == fmt3.pageNumber())


compareOutput ("format_testfile2", "./format_test.pl 2", nil)



# test FormatExcpetion throwing
begin
  fmt3.setTop(fmt3)
  raise "testing error"
rescue FormatException
end

bad_format = <<DOT
 @<<< @>>
 name,
DOT

begin
  badFormat = Format.new(bad_format)
  raise "testing error"
rescue  FormatException
end



## New test for # and <, perl test #2
top_ex = <<DOT
   Piggy Locations for @<< @#, @###
                     month, day, year

Number: location              toe size
-------------------------------------------
DOT

ex = <<DOT
@)  ~~  @<<<<<<<<<<<<<<<<       @#.##
num,    location,             toe_size
DOT

bot_ex = <<BOT
-------------------------------------------
BOT

File.open("format_testfile3", File::CREAT | File::WRONLY | File::TRUNC) { |file|
  top_fmt = Format.new (top_ex) 
  body_fmt = Format.new (ex)
  bottom_fmt = Format.new (bot_ex)
  body_fmt.setTop(top_fmt)
  body_fmt.io = file

  month = "Sep"
  day = 18
  year = 2001
  num = 1
  body_fmt.setPageLength(11)
  ["Market", "Home", "Eating Roast Beef", "Having None", "On the way home"].each {|location|
    toe_size = (num * 3.5)
    body_fmt.printFormat(binding)
    num += 1
  }
  raise "line numbering is broken #{body_fmt.pageNumber()}" unless (1 == body_fmt.pageNumber())
}

compareOutput ("format_testfile3", "format_test.pl 3", nil)

## hand check for bottoms, this should be files of 5 piggy lines plus a header and a footer.
File.open("format_testfile4", File::CREAT | File::WRONLY | File::TRUNC) { |file|
  top_fmt = Format.new (top_ex) 
  body_fmt = Format.new (ex)
  bottom_fmt = Format.new (bot_ex)
  body_fmt.setTop(top_fmt)
  body_fmt.setBottom(bottom_fmt)
  body_fmt.io = file
  body_fmt.setPageLength(10)
  month = "Sep"
  day = 18
  year = 2001
  10.times do 
    #day += 1
    num = 1
    ["Market", "Home", "Eating Roast Beef", "Having None", "On the way home"].each {|location|
      toe_size = (num * 3.5)
      body_fmt.printFormat(binding)
      num += 1
    }
  end
  body_fmt.finishPageWithoutFF(binding)
}

compareOutput ("format_testfile4", "format_test.pl 4", nil)

## hand check for finishPage, this should be some formats followed by blank lines and a footer
File.open("format_testfile5", File::CREAT | File::WRONLY | File::TRUNC) { |file|
  top_fmt = Format.new (top_ex) 
  body_fmt = Format.new (ex)
  bottom_fmt = Format.new (bot_ex)
  body_fmt.setTop(top_fmt)
  body_fmt.setBottom(bottom_fmt)
  body_fmt.io = file
  body_fmt.setPageLength(20)
  month = "Sep"
  day = 19
  year = 2001
  count = 0
  10.times do 
    count += 1
    num = 1
    ["Market", "Home", "Eating Roast Beef", "Having None", "On the way home"].each {|location|
      puts "the page number is #{body_fmt.pageNumber()}" if (body_fmt.pageNumber() == nil)
      toe_size = 1
      body_fmt.printFormat(binding)
      num += 1
    }
    body_fmt.finishPageWithFF (binding) unless (count == 10)
  end
  body_fmt.finishPageWithoutFF (binding)
}

compareOutput ("format_testfile5", "format_test.pl 5", nil)

## Examples of ^
hat_ex = <<BOT
Format that uses the hat
@|| ^<< ^||
long, long, long
^|||| @<<<< ^|||| ^###.## ^###.## ^###.#
long, long, long, num, num, num
Run out of chars!
^|||| ^<<<< ^|||| --
short, short,short
^<<<*^<<<<*^<<<<<<<<<<<<<
long, long, long
BOT

empty_top = <<TOP
TOP

hat_top = Format.new (empty_top)

hat_format = Format.new (hat_ex)
hat_format.setTop(hat_top)
long = "01234567890abcde fg  h i jklmn12345678901234567890123456.789"
num = 123.456
short = '123.456'

File.open("format_testfile6", File::CREAT | File::WRONLY | File::TRUNC) { |file|
  hat_format.io = file
  hat_format.printFormat(binding)
  raise "page numbering not working " unless (1 == hat_format.pageNumber())
}

compareOutput ("format_testfile6", "format_test.pl 6", nil)

test_5 = ["~  non_blank: @<<<|"]
test_5.push ("non_blank")
test_5.push ("~ blank unless $blank holds something, which it doesn\'t @<<<")
test_5.push ("blank")
test_5.push ("~~ repeat: ^<<.")
test_5.push ("string")

###test_5 = <<T5
###~  non_blank: @<<<|
###non_blank
###~ blank unless $blank holds something, which it doesn't @<<<
###blank
###~~ repeat: ^<<.
###string
###T5

File.open("format_testfile7", File::CREAT | File::WRONLY | File::TRUNC) { |file|
  tilde_format = Format.new (test_5)
  blank = ""
  non_blank = "  "
  string = "abcdefghijklmn"
  tilde_format.io = file
  tilde_format.printFormat(binding)
}
                            

compareOutput ("format_testfile7", "format_test.pl 7", nil)


#Problems when formats don't fit on a single page!
File.open("format_testfile8", File::CREAT | File::WRONLY | File::TRUNC) { |file|
  tilde_format = Format.new (test_5)
  blank = ""
  non_blank = "  "
  string = ("a"*100) + "b"
  tilde_format.io = file
  tilde_format.setPageLength (10)    
  tilde_format.printFormat(binding)
  tilde_format.finishPageWithoutFF (binding)
}

compareOutput ("format_testfile8", "format_test.pl 8", nil)



puts "Test successful"
